#include "mouse.h"
#include "bridge.h"
#include "../common/sources/system/cgraphics2d.h"
#include "../common/sources/system/filecache.h"

CMouse  mouse;

//-----------------------------------------------------------------------------------------
CMouse::CMouse()
{
    mPos = CVec2();
    mButtonPressing = false;
    mButtonReleasing = false;
    mButtonPressed = false;
}

//-----------------------------------------------------------------------------------------
void CMouse::update()
{
    mPos = bridge.mMousePos;
    mButtonPressing = (mButtonPressed == false && bridge.mMouseButton == true);
    mButtonReleasing = (mButtonPressed == true && bridge.mMouseButton == false);
    mButtonPressed = bridge.mMouseButton;
}

//-----------------------------------------------------------------------------------------
void CMouse::clearAreas()
{
    mAreaList.clear();
}

//-----------------------------------------------------------------------------------------
void CMouse::addArea(EMouseAreaTypes type, CRect rect, ECursorTypes cursor, void* param1, void* param2)
{
    SMouseArea  newArea;
    newArea.type = type;
    newArea.rect = rect;
    newArea.param1 = param1;
    newArea.param2 = param2;
    newArea.cursor = cursor;

    mAreaList.push_back(newArea);
}

//-----------------------------------------------------------------------------------------
SMouseArea* CMouse::clickedArea(QImage* image)
{
    SMouseArea*  area = NULL;

    for (int i = mAreaList.size() - 1; i >= 0; --i)
    {
        CRect*  rect = &mAreaList[i].rect;

        if (mPos.x >= rect->tl.x &&
            mPos.y >= rect->tl.y &&
            mPos.x <= rect->br.x &&
            mPos.y <= rect->br.y)
        {
            area = &mAreaList[i];
            break;
        }
    }

    if (area == NULL || area->cursor != eCursor_None)
    {
        std::string fileName;

        if (area == NULL)
            fileName = "gfx/interface/CursorArrow.png";
        else if (area->cursor == eCursor_Arrow)
            fileName = "gfx/interface/CursorArrow.png";
        else
            fileName = "gfx/interface/CursorHand.png";

        QImage  cursor = fileCache.getImage(fileName);
        graph2D.drawImage(image, mPos, cursor);
    }

    if (mButtonPressed == true)
    {
        return area;
    }
    return NULL;
}
